module OO
  class Configuration
    attr_reader :licence_max_depth, :default_licence_keys,
                :licence_missing_reaction, :licence_checkers_default_user,
                :licence_checkers_default_key

    def initialize
      @licence_max_depth = 4
      @default_licence_keys = []
      @licence_missing_reaction =
        -> { :licence_missing_reaction_not_configured }

      @licence_checkers_default_user =
        -> { :licence_checkers_default_user_not_configured }

      @licence_checkers_default_key =
        -> { :licence_checkers_default_key_not_configured }
    end

    def licence_max_depth=(depth)
      raise Errors::LicenceMaxDepthInvalid unless depth.is_a?(Fixnum) &&
                                                  depth > 0
      @licence_max_depth = depth
    end

    def default_licence_keys=(licence_keys)
      raise Errors::DefaultLicenceKeysInvalid unless licence_keys.is_a?(Array)
      @default_licence_keys = licence_keys
    end

    def licence_missing_reaction=(action)
      raise Errors::LicenceMissingReactionInvalid unless action.is_a?(Proc)
      @licence_missing_reaction = action
    end

    def licence_checkers_default_user=(action)
      raise Errors::LicenceCheckersDefaultUserInvalid unless action.is_a?(Proc)
      @licence_checkers_default_user = action
    end

    def licence_checkers_default_key=(action)
      raise Errors::LicenceCheckersDefaultKeyInvalid unless action.is_a?(Proc)
      @licence_checkers_default_key = action
    end
  end
end
